@@ -8,7 +8,7 @@ from typing import List
 
 from .SharePointUser import SharePointUser
 from .SharePointUserList import SharePointUserList
-from .SharePointListItem import SharePointListItem
+from .SharePointListItem import SharePointListItem, SharepointSiteCase
 from .SharePointList import SharePointList, CasesList, TimeRegistrationList
 from .SharePointLists import SharePointLists
 
@@ -64,7 +64,12 @@ class SharePointAPI:
         r = requests.post(url,
                          auth=HttpNtlmAuth(self.username, self.password), headers=headers, json=post_data, proxies=self.proxies)
 
-        if r.status_code == 404:
+        if r.status_code == 400:
+            print('Failed to connect: ', r.status_code)
+            print(r.text)
+            print('Exiting.')
+            raise ConnectionError
+        elif r.status_code == 404:
             print('Failed to connect: ', r.status_code)
             print(r.request.url)
             print(r.request.body)
@@ -426,6 +431,24 @@ class SharePointAPI:
             top: Maximum items to query from the list
         '''
         return self.get_list_by_name(sharepoint_site, sp_list_name, filters, top, view_path, SPListType=CasesList)
+    
+    def get_case(self, sharepoint_site, sp_list, item_id) -> SharePointListItem:
+        '''
+            Returns a list of users from a given sharepoint_site
+        '''
+
+        if isinstance(sp_list, SharePointList):
+            guid = sp_list.guid
+
+        elif isinstance(sp_list, str):
+            guid = sp_list
+        else:
+            sys.exit(1)
+
+        r = self._api_get_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/Lists(guid\'{guid}\')/items({item_id})')
+
+        settings = r.json()["d"]
+        return SharepointSiteCase(self, sharepoint_site, guid, settings)
 
     # Time Registration
 
@@ -465,13 +488,46 @@ class SharePointAPI:
         return self.get_list_by_name(sharepoint_site, sp_list_name, filters, top, view_path, SPListType=TimeRegistrationList)
 
     # SP Files
+
+    def folder_exists(self, sharepoint_site, folder, in_doc_lib=True):
+        '''
+            Returns a boolean to indicate whether a folder exists
+        '''
+
+        if in_doc_lib:
+            folder = os.path.join("DocumentLibrary", folder)
+
+        r = self._api_get_call(f"{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/GetFolderByServerRelativeUrl('/cases/{sharepoint_site}/{folder}')/ListItemAllFields")
+
+        return False if ("ListItemAllFields" in r.json()["d"] and r.json()["d"]["ListItemAllFields"] is None) else True
+
+    def create_new_folder(self, sharepoint_site, folder, new_folder, in_doc_lib=True):
+        '''
+            Creates a new folder
+        '''
+
+        if in_doc_lib:
+            folder = os.path.join("DocumentLibrary", folder)
+
+        r = self._api_post_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/contextinfo', {})
+        form_digest_value = r.json()["d"]["GetContextWebInformation"]["FormDigestValue"]
+
+        r = self._api_post_call(f"{self.sharepoint_url}/cases/{sharepoint_site}/_api/web/folders",
+                form_digest_value=form_digest_value,
+                post_data={
+                    "ServerRelativeUrl": f'/cases/{sharepoint_site}/{folder}/{new_folder}',
+                }
+            )
+
+        return r.status_code
+
     def get_files(self, sharepoint_site, folder, in_doc_lib=True):
         '''
             Returns a list of files from a folder in a given sharepoint_site
         '''
 
         if in_doc_lib:
-            folder = "DocumentLibrary/" + folder
+            folder = os.path.join("DocumentLibrary", folder)
 
         r = self._api_get_call(f"{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/GetFolderByServerRelativeUrl('/cases/{sharepoint_site}/{folder}')/Files")
 
@@ -483,7 +539,7 @@ class SharePointAPI:
         '''
 
         if in_doc_lib:
-            folder = "DocumentLibrary/" + folder
+            folder = os.path.join("DocumentLibrary", folder)
 
         r = self._api_get_call(f"{self.sharepoint_url}/cases/{sharepoint_site}/_api/Web/GetFolderByServerRelativeUrl('/cases/{sharepoint_site}/{folder}')/Files('{file}')/$value")
         return r._content
@@ -495,7 +551,7 @@ class SharePointAPI:
         '''
 
         if in_doc_lib:
-            folder = "DocumentLibrary/" + folder
+            folder = os.path.join("DocumentLibrary", folder)
 
         if not out_file:
             out_file = file
@@ -511,7 +567,7 @@ class SharePointAPI:
         '''
 
         if in_doc_lib:
-            folder = "DocumentLibrary/" + folder
+            folder = os.path.join("DocumentLibrary", folder)
 
         r = self._api_post_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/contextinfo', {})
         form_digest_value = r.json()["d"]["GetContextWebInformation"]["FormDigestValue"]
@@ -528,7 +584,7 @@ class SharePointAPI:
         '''
 
         if in_doc_lib:
-            folder = "DocumentLibrary/" + folder
+            folder = os.path.join("DocumentLibrary", folder)
 
         if not in_file:
             in_file = file
@@ -543,7 +599,7 @@ class SharePointAPI:
         '''
 
         if in_doc_lib:
-            folder = "DocumentLibrary/" + folder
+            folder = os.path.join("DocumentLibrary", folder)
 
         r = self._api_post_call(f'{self.sharepoint_url}/cases/{sharepoint_site}/_api/contextinfo', {})
         form_digest_value = r.json()["d"]["GetContextWebInformation"]["FormDigestValue"]
@@ -560,7 +616,7 @@ class SharePointAPI:
         '''
 
         if in_doc_lib:
-            folder = "DocumentLibrary/" + folder
+            folder = os.path.join("DocumentLibrary", folder)
 
         if not out_file:
             out_file = 'copy of '+file
